import logging
import os
import pandas as pd
import geopandas as gpd
from shapely.geometry import shape



def get_field_soil_data(geometry: dict) -> pd.DataFrame:
    logging.info(f"Creating soil data")
    soil_id = get_soil_id(geometry=geometry)
    return load_soil_data(soil_id=soil_id)


def get_soil_id(geometry: dict) -> int:
    """
    Retrieve the soil ID of the geometry. This is done based on a soil data shapefile provided by BCGMS.
    The intersection of the given geometry with the shapefile data is used to determine the soil id
    :param geometry: Geometry for which to get the soil ID
    :return: ID of the soil
    """
    logging.info(f"Retrieving the soil id from the BCGMS shapefile")
    geom_shape = shape(geometry)
    soil_data = gpd.read_file(
        os.getcwd() + "/grassat/data/soil/shapefiles/bcgms/BCGMS_soil.shp"
    ).to_crs(epsg=4326)
    matches = soil_data[soil_data.intersects(geom_shape)]
    if matches.empty:
        raise Exception(f"Could not find soil data for geometry")
    return matches.iloc[0]["SOIL_GROUP"]


def load_soil_data(soil_id) -> pd.DataFrame:
    logging.info(f"Loading soil data for soil with id {soil_id}")
        
    # Fetch the curve number (cn), readily evaporable water (rew), and soil layers information.
    cn, rew, layers = read_soil_layers(soil_id=soil_id)
    
    # Transform the 'layers' list into a DataFrame with the required format
    data = {
        'sdpt': [layers[0]['thickness'] * 1000],  # Convert thickness from m to mm TODO need to double check if we take it from soil file or just set it to a constant value
        'mcfc': [layers[0]['fc']],               
        'mcsat': [layers[0]['sat']],             
        'mcwp': [layers[0]['wp']],               
    }
    df = pd.DataFrame(data)
    return df


def read_soil_layers(soil_id: int) -> tuple:
    """
    Read the soil configuration file from disk.
    :param soil_id: ID of the soil configuration to read
    :return: Tuple containing the CN, REW and soil layers
    """
    layers = list()
    logging.info(f"Reading soil layer for soil with id {soil_id}")
    with open(
        os.getcwd() + "/grassat//data/soil/config/SOIL_%03d.SOL" % int(soil_id), "r"
    ) as soil_file:
        lines = soil_file.readlines()
        cn = float(lines[2].split(":")[0])
        rew = float(lines[3].split(":")[0])
        for line in lines[8:]:
            parts = list(filter(lambda x: x != "", line.split(" ")))
            layers.append(
                {
                    "thickness": float(parts[0]),
                    "sat": float(parts[1]) / 100,
                    "fc": float(parts[2]) / 100,
                    "wp": float(parts[3]) / 100,
                    "ksat": float(parts[4]),
                    "cra": float(parts[5]),
                    "crb": float(parts[6]),
                }
            )
        soil_file.close()
    return cn, rew, layers
