'''
python -m unittest tests/grassat_model_test.py
'''

import unittest
from unittest.mock import patch
from grassat.database.meteo import MeteoDatabase
from grassat.model.field_data import *
from pathlib import Path
import json
import unittest
from datetime import datetime
from fastapi.testclient import TestClient
from grassat.main import app   # Ensure this is your FastAPI app's module


class TestGrassatModel(unittest.TestCase):
    
    @patch("grassat.database.base.Elasticsearch")  # Mock the Elasticsearch client class
    def setUp(self, MockElasticsearch):
        # Initialize the FastAPI test client
        self.client = TestClient(app)
        
         # Mock Elasticsearch instance
        self.mock_es = MockElasticsearch.return_value
        
        # Initialize MeteoDatabase after mocking Elasticsearch
        self.meteo_db = MeteoDatabase()
        

    def test_run_model_with_empty_irrigations_and_fertilisations(self):
        # Load mock request and expected response from a JSON file as a dictionary
        with open(Path(__file__).parent.parent / "tests" / "fixtures" / "grassatRequest1.json", 'r') as file:
            self.mock_request_body = json.load(file)
            
        with open(Path(__file__).parent.parent / "tests" / "fixtures" / "expectedResponse1.json", 'r') as file:
            self.expectedResponse = json.load(file)
            
        # Send POST request to the endpoint
        response = self.client.post("/model", json=self.mock_request_body)

        # Validate the response
        self.assertEqual(response.status_code, 200)

        # Assert that the response matches the expected response
        response_json = response.json()
        self.assertEqual(response_json, self.expectedResponse)
        
    def test_run_model_with_irrigations_and_fertilisations(self):
        # Load mock request and expected response from a JSON file as a dictionary
        with open(Path(__file__).parent.parent / "tests" / "fixtures" / "grassatRequest2.json", 'r') as file:
            self.mock_request_body = json.load(file)
            
        with open(Path(__file__).parent.parent / "tests" / "fixtures" / "expectedResponse2.json", 'r') as file:
            self.expectedResponse = json.load(file)
        
        # Send POST request to the endpoint
        response = self.client.post("/model", json=self.mock_request_body)

        # Validate the response
        self.assertEqual(response.status_code, 200)
        
        # Assert that the response matches the expected response
        response_json = response.json()
        self.assertEqual(response_json, self.expectedResponse)
        
    def test_run_model_with_current_year_start_date(self):
        # Load mock request and expected response from a JSON file as a dictionary
        with open(Path(__file__).parent.parent / "tests" / "fixtures" / "grassatRequest3.json", 'r') as file:
            self.mock_request_body = json.load(file)
            
        # Get the current year and dayOfYear (DOY)
        current_year = datetime.now().year
        dayOfYear = datetime.now().timetuple().tm_yday

        # Update startDate and endDate in mock_request_body
        self.mock_request_body['grassatRequest']['startDate'] = f"{current_year}-01-01T12:00:00.000Z"
        self.mock_request_body['grassatRequest']['endDate'] = f"{current_year}-12-31T12:00:00.000Z"
                   
        # Send POST request to the endpoint
        response = self.client.post("/model", json=self.mock_request_body)

        # Validate the response
        self.assertEqual(response.status_code, 200)
        
        # Check if the response matches the expected structure
        response_json = response.json()
        self.assertIn("Weight of living stems; WST", response_json)
        self.assertIn("Harvestable leaf weight; HRVBL", response_json)
        self.assertIn("Harvest date", response_json)
        self.assertIn("Date", response_json)
        self.assertIn("Fertilizer N application; (0=NO, 1=YES)", response_json)
        self.assertIn("N concentration in total vegetative living above ground biomass", response_json)
        self.assertIn("Total harvested weight", response_json)

        # Get the length of response property arrays
        wst_length = len(response_json["Weight of living stems; WST"])
        hrvbl_length = len(response_json["Harvestable leaf weight; HRVBL"])
        harvest_date_length = len(response_json["Harvest date"])
        date_length = len(response_json["Date"])
        fertilizer_length = len(response_json["Fertilizer N application; (0=NO, 1=YES)"])

        # Meteo data will be fetched until yesterday or previuos day at currrent year, 
        # so length of response property arrays should be equal to dayOfYear-1 or dayOfYear-2
        self.assertIn(
            wst_length,
            [dayOfYear-1, dayOfYear-2],
            f"DOY ({dayOfYear-1}) or ({dayOfYear-2}) does not match the length of WST array ({wst_length})."
        )
        self.assertIn(
            hrvbl_length,
            [dayOfYear-1, dayOfYear-2],
            f"DOY ({dayOfYear-1}) or ({dayOfYear-2}) does not match the length of HRVBL array ({hrvbl_length})."
        )
        self.assertIn(
            harvest_date_length,
            [dayOfYear-1, dayOfYear-2],
            f"DOY ({dayOfYear-1}) or ({dayOfYear-2}) does not match the length of Harvest date array ({harvest_date_length})."
        )
        self.assertIn(
            date_length,
            [dayOfYear-1, dayOfYear-2],
            f"DOY ({dayOfYear-1}) or ({dayOfYear-2}) does not match the length of Date array ({date_length})."
        )
        self.assertIn(
            fertilizer_length,
            [dayOfYear-1, dayOfYear-2],
            f"DOY ({dayOfYear-1}) or ({dayOfYear-2}) does not match the length of Fertilizer N application array ({fertilizer_length})."
        )
        
    def test_no_meteo_data(self):
        """
        Test case where no meteo data is found for the geometry
        """
        # Load mock request from a JSON file as a dictionary
        with open(Path(__file__).parent.parent / "tests" / "fixtures" / "grassatRequest1.json", 'r') as file:
            self.mock_request_body = json.load(file)
                  
        field_data: FieldData = FieldData.from_request(self.mock_request_body['grassatRequest'])
        
        # Elasticsearch search query result - no hits
        self.mock_es.search.return_value = {"hits": {"hits": []}}

        # Test whether exception is raised when no meteo data is found
        with self.assertRaises(Exception) as context:
            self.meteo_db.get_meteo(field_data)

        # Check that the exception message is correct
        self.assertTrue(
            "Could not find meteo information for geometry" in str(context.exception)
        )



if __name__ == '__main__':
    unittest.main()
